import sharp from 'sharp';
import type { SharpOptions, ResizeOptions } from 'sharp';
import { Hashes } from '../hash';
import { join, extname } from 'path';
import { tmpdir } from 'os';
import { mkdir, unlink } from 'fs/promises';
import { Shell } from '../shell';

export class Image {
	private prefix : string = 'fm_thumbs';
	private thumbnailCache : Buffer = null;
	private thumbnailHash : string = null;
	private blankCache : Buffer = null;
	private blankHash : string = null;
	private tmp : string = null;

	constructor () {
		if (this.tmp === null) {
			this.tmp = tmpdir();
		}
	}

	private async mktemp (ext : string = '.png') : Promise<string> {
		const randomString : string = Math.random().toString(36).slice(2);
		const tempPath : string = join(this.tmp, this.prefix, randomString + `.${ext}`);

		try {
			await mkdir(join(this.tmp, this.prefix));
		} catch (err) {
			//
		}
		return tempPath;
	}

	private async dpx2png (input : string, output : string) {
		let dpx : any = null;
		const args : string[] = [
			'convert',
			input,
			'-colorspace',  'RGB',
			'-depth', '8',
			output
		];
		const shell : Shell = new Shell(args, null, null, null, true);
		try {
			await shell.execute();
		} catch (err) {
			//
		}
	}

	public async thumbnail (path : string, width : number, height: number) : Promise<Buffer>{
		const hash : string = Hashes.stringHash(`${path},${width},${height}`);
		let newPath : string = null;
		if (hash !== this.thumbnailHash) {
			if (extname(path).toLowerCase() === '.dpx') {
				try {
					newPath = await this.mktemp('png');
					await this.dpx2png(path, newPath);
					path = newPath;
				} catch (err) {
					//
				}
			}
			const options : ResizeOptions = {
				width,
				height,
				fit: sharp.fit.fill
			}
			this.thumbnailCache = await sharp(path).resize(options).jpeg().toBuffer();
			this.thumbnailHash = hash;
			if (newPath !== null) {
				try {
					await unlink(newPath);
				} catch (err) {
					//
				}
			}
		}
		return this.thumbnailCache;
	}

	public async blank (width : number, height : number) {
		const hash : string = Hashes.stringHash(`${width},${height}`);
		if (hash !== this.blankHash) {
			const options : SharpOptions = {
				create: {
					width,
					height,
					channels: 3,
					background: { r: 125, g: 125, b: 125 }
				}
			};
			this.blankCache = await sharp(options).jpeg().toBuffer();
			this.blankHash = hash;
		}
		return this.blankCache;
	}
}

module.exports = { Image };