import nodemailer from 'nodemailer';
import { createLog } from '../log';
import type { Logger } from 'winston';

const from : string = process.env['MAIL_FROM'];
const smtp : string = process.env['MAIL_SMTP'];
const port : number = typeof process.env['MAIL_SMTP_PORT'] !== 'undefined' ? 25 : parseInt(process.env['MAIL_SMTP_PORT'], 10);

let transporter : nodemailer.Transporter = null;

const log : Logger = createLog('mail');

if (typeof from !== 'undefined' && from !== null && from !== '' &&
    typeof smtp !== 'undefined' && smtp !== null && smtp !== '') {
    transporter = nodemailer.createTransport({
        host: smtp,
        port,
        secure : false,
        tls: { rejectUnauthorized: false }
    });
}

export async function sendMail (to : string, subject : string, body : string) : Promise<boolean>{
    if (transporter !== null) {
        try {
            await transporter.sendMail({
                from,
                to,
                subject,
                html: body
            });
            log.info(`Sent email "${subject}"`);
        } catch (err) {
            log.error(`Error sending email "${subject}"`);
            log.error(err);
            return false;
        }
    } else {
        log.warn(`Email not configured, not sending "${subject}"`);
        return false;
    }
    return true;
}

module.exports = { sendMail };